package realname

import (
	"crypto/hmac"
	"crypto/sha1"
	"encoding/base64"
	"fmt"
	"net/http"
	"time"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/renzhengshouquan"
	"shylinux.com/x/community/src/renzhengshouquan/model"
)

type realname struct {
	renzhengshouquan.Table
	order  string `data:"18"`
	fields string `data:"source_id,secret_id,secret_key,user_uid"`
	create string `name:"create source_id* secret_id* secret_key*" role:"worker"`
	remove string `name:"remove" role:"worker"`
	list   string `name:"list auth_uid uid auto" role:"worker"`
}

func (s realname) List(m *ice.Message, arg ...string) {
	s.ValueList(m, arg).Action().Display("")
}
func init() { ice.TeamCtxCmd(realname{}) }

func init() { renzhengshouquan.RealnameVendor = realname{} }

func (s realname) Check(m *ice.Message, arg ...string) {
	msg := s.Select(m.Spawn(), model.AUTH_UID, s.GetAuthUID(m))
	source := msg.Append("source_id")
	secretId := msg.Append("secret_id")
	secretKey := msg.Append("secret_key")
	auth, datetime := calcAuthorization(source, secretId, secretKey)
	m.Option(web.SPIDE_HEADER, map[string]string{"X-Source": source, "Authorization": auth, "X-Date": datetime})
	res := m.Cmdx(web.SPIDE, ice.DEV, web.SPIDE_RAW, http.MethodPost, "https://service-rbgpp2hy-1305308687.gz.apigw.tencentcs.com/release/release/mobile/3-realnameauth",
		web.SPIDE_FORM, "mobile_number", arg[0], "idcard_number", arg[1], "name", arg[2])
	m.WarnNotRight(kit.Value(kit.UnMarshal(res), "data.result") != "0")
}

func calcAuthorization(source string, secretId string, secretKey string) (auth string, datetime string) {
	timeLocation, _ := time.LoadLocation("Etc/GMT")
	datetime = time.Now().In(timeLocation).Format("Mon, 02 Jan 2006 15:04:05 GMT")
	signStr := fmt.Sprintf("x-date: %s\nx-source: %s", datetime, source)
	mac := hmac.New(sha1.New, []byte(secretKey))
	mac.Write([]byte(signStr))
	sign := base64.StdEncoding.EncodeToString(mac.Sum(nil))
	auth = fmt.Sprintf("hmac id=\"%s\", algorithm=\"hmac-sha1\", headers=\"x-date x-source\", signature=\"%s\"", secretId, sign)
	return auth, datetime
}
